#sc-TissueMapper
#Author: Gervaise H. Henry
#Email: gervaise.henry@utsouthwestern.edu
#Lab: Strand Lab, Deparment of Urology, University of Texas Southwestern Medical Center


scLoad <- function(p,d=1,mc=3,mg=200,sub=FALSE){
  #Load and prefilter filtered_gene_bc_matrices_mex output from cellranger
  
  #Inputs:
  #p = project name
  #d = column of demultiplex.csv file to use to subset samples
  #mc = minimum cells (filter out genes which occur in fewer cells)
  #mg = minimum genes (filter out cells which conntain fewer genes)
  #sub = multi-dataset experiment  

  #Outputs:
  #Seurat object
  
  #Load filtered_gene_bc_matrices_mex output from cellranger
  if (sub==FALSE){
    sc10x.data <- Read10X(data.dir="./analysis/DATA/10x/filtered_gene_bc_matrices_mex/GRCh38/")
  } else {
    sc10x.data <- Read10X(data.dir=paste0("./analysis/DATA/",p,"/10x/filtered_gene_bc_matrices_mex/GRCh38/"))
  }
  sc10x <- new("seurat",raw.data=sc10x.data)
  
  #Label sample names from aggregation_csv.csv
  if (sub==FALSE){
    sc10x.aggr <- read_csv("./analysis/DATA/10x/aggregation_csv.csv")
  } else {
    sc10x.aggr <- read_csv(paste0("./analysis/DATA/",p,"/10x/aggregation_csv.csv"))
  }
  cell.codes <- as.data.frame(sc10x@raw.data@Dimnames[[2]])
  colnames(cell.codes) <- "barcodes"
  rownames(cell.codes) <- cell.codes$barcodes
  cell.codes$lib.codes <- as.factor(gsub(pattern=".+-",replacement="",cell.codes$barcodes))
  cell.codes$samples <- sc10x.aggr$library_id[match(cell.codes$lib.codes,as.numeric(rownames(sc10x.aggr)))]
  sc10x <- CreateSeuratObject(raw.data=sc10x.data,meta.data=cell.codes["samples"],min.cells=mc,min.genes=mg,project=p)
  
  #Create groups found in demultiplex.csv
  if (sub==FALSE){
    sc10x.demultiplex <- read_csv(paste0("./analysis/DATA/",p,"-demultiplex.csv"))
  } else {
    sc10x.demultiplex <- read_csv(paste0("./analysis/DATA/",p,"/",p,"-demultiplex.csv"))

  }
  for (i in 2:ncol(sc10x.demultiplex)){
    sc10x <- SetAllIdent(object=sc10x,id="samples")
    merge.cluster <- apply(sc10x.demultiplex[,i],1,as.character)
    merge.cluster[merge.cluster==1] <- colnames(sc10x.demultiplex[,i])
    sc10x@ident <- plyr::mapvalues(x=sc10x@ident,from=sc10x.demultiplex$Samples,to=merge.cluster)
    sc10x <- StashIdent(object=sc10x,save.name=colnames(sc10x.demultiplex[,i]))
  }
  
  return(sc10x)
}


scSubset <- function(sc10x,i="ALL",g="ALL"){
  #Subset cells based on an identity
  
  #Inputs:
  #sc10x = seruat object
  #i = identity to use
  #g = group to subset by
  
  #Outputs:
  #Seurat object
  
  sc10x <- SetAllIdent(object=sc10x,id=i)
  sc10x.sub <- SubsetData(object=sc10x,ident.use=g)

  return(sc10x.sub)
}


scCellCycle <- function(sc10x,sub=FALSE){
  #Runs Seurat based PCA analysis for cell cycle ID
  
  #Inputs:
  #sc10x = Seruat object
  
  #Outputs:
  #results[1] = Seurat object
  #results[2] = s genes
  #results[3] = g2m genes
  
  #Make sub-folders if necessary
  if (sub==FALSE){
    folder <- "./analysis/qc/cc/"
  } else {
    folder <- paste0("./analysis/qc/cc/",sub,"/")
    if (!dir.exists(folder)){
      dir.create(folder)
  }}
  
  #score cell cycle and generate figures
  genes.cc <- readLines(con="./genesets/regev_lab_cell_cycle_genes.txt")
  genes.s <- genes.cc[1:43]
  genes.g2m <- genes.cc[44:97]
  sc10x <- NormalizeData(object=sc10x)
  sc10x <- ScaleData(object=sc10x,display.progress=FALSE,do.par=TRUE,num.cores=45)
  sc10x <- CellCycleScoring(object=sc10x,s.genes=genes.s,g2m.genes=genes.g2m,set.ident=TRUE)
  
  postscript(paste0(folder,"Ridge_cc.Raw.eps"))
  plot <- RidgePlot(object=sc10x,features.plot=c("PCNA","TOP2A","MCM6","MKI67"),y.log=TRUE,nCol=2,do.return=TRUE)
  plot(plot)
  dev.off()
  postscript(paste0(folder,"Violin_cc.Raw.eps"))
  plot <- VlnPlot(object=sc10x,features.plot=c("PCNA","TOP2A","MCM6","MKI67"),nCol=2,point.size.use=1,size.title.use=20,x.lab.rot=TRUE)
  plot(plot)
  dev.off()
  sc10x <- RunPCA(object=sc10x,pc.genes=c(genes.s,genes.g2m),do.print=FALSE,pcs.store=2)
  postscript(paste0(folder,"PCA_cc.Raw.eps"))
  plot <- PCAPlot(object=sc10x,do.return=TRUE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  gc()
  sc10x <- ScaleData(object=sc10x,vars.to.regress=c("S.Score","G2M.Score"),display.progress=FALSE,do.par=TRUE,num.cores=45)
  gc()
  sc10x <- RunPCA(object=sc10x,pc.genes=c(genes.s,genes.g2m),do.print=FALSE,pcs.store=2)
  postscript(paste0(folder,"PCA_cc.Norm.eps"))
  plot <- PCAPlot(object=sc10x,do.return=TRUE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  
  results <- list(
    sc10x=sc10x,
    genes.s=genes.s,
    genes.g2m=genes.g2m
  )
  return(results)
}


scQC <- function(sc10x,lg=500,hg=2500,lm=-Inf,hm=0.1,sub=FALSE){
  #QC and filter Seurat object

  #Inputs:
  #sc10x = Seruat object
  #lf = threshold for cells with minimum genes
  #hg = threshold for cells with maximum genes
  #lm = threshold for cells with minimum %mito genes
  #hm = threshold for cells with maximum %mito genes
  
  #Outputs:
  #result[1] = Seurat object
  #result[2] = raw cell count
  #result[3] = raw gene count
  #result[4] = filtered cell count
  #result[5] = filtered gene count
  
  #Make sub-folders if necessary
  if (sub==FALSE){
    folder <- "./analysis/qc/"
  } else {
    folder <- paste0("./analysis/qc/",sub,"/")
    if (!dir.exists(folder)){
      dir.create(folder)
  }}

  #Calculate %mito genes
  mito.genes <- grep(pattern="^MT-",x=rownames(x=sc10x@data),value=TRUE)
  percent.mito <- colSums(as.matrix(sc10x@raw.data[mito.genes,]))/colSums(as.matrix(sc10x@raw.data))
  sc10x <- AddMetaData(object=sc10x,metadata=percent.mito,col.name="percent.mito")
  
  #Plot raw stats
  sc10x <- SetAllIdent(object=sc10x,id="ALL")
  postscript(paste0(folder,"Violin_qc.raw.eps"))
  plot <- VlnPlot(object=sc10x,features.plot=c("nGene","nUMI","percent.mito"),nCol=3,do.return=TRUE)
  plot(plot)
  dev.off()
  postscript(paste0(folder,"Plot_qc.raw.eps"))
  par(mfrow=c(1,2))
  GenePlot(object=sc10x,gene1="nUMI",gene2="percent.mito")
  GenePlot(object=sc10x,gene1="nUMI",gene2="nGene")
  dev.off()
  counts.cell.raw <- length(sc10x@data@Dimnames[[2]])
  counts.gene.raw <- length(sc10x@data@Dimnames[[1]])
  
  #Filter/normalize data
  sc10x <- FilterCells(object=sc10x,subset.names=c("nGene","percent.mito"),low.thresholds=c(lg,lm),high.thresholds=c(hg,hm))
  sc10x <- NormalizeData(object=sc10x,normalization.method="LogNormalize",scale.factor=10000)
  
  #Plot filtered stats
  postscript(paste0(folder,"Violin_qc.filtered.eps"))
  plot <- VlnPlot(object=sc10x,features.plot=c("nGene","nUMI","percent.mito"),nCol=3,do.return=TRUE)
  plot(plot)
  dev.off()
  postscript(paste0(folder,"Plot_qc.filtered.eps"))
  par(mfrow=c(1,2))
  GenePlot(object=sc10x,gene1="nUMI",gene2="percent.mito")
  GenePlot(object=sc10x,gene1="nUMI",gene2="nGene")
  dev.off()
  counts.cell.filtered <- length(sc10x@data@Dimnames[[2]])
  counts.gene.filtered <- length(sc10x@data@Dimnames[[1]])
  
  #Remove mito genes
  sc10x@data <- sc10x@data[!rownames(sc10x@data) %in% mito.genes,]
    
  results <- list(
    sc10x=sc10x,
    counts.cell.raw=counts.cell.raw,
    counts.gene.raw=counts.gene.raw,
    counts.cell.filtered=counts.cell.filtered,
    counts.gene.filtered=counts.gene.filtered
  )
  return(results)
}


scPC <- function(sc10x,lx=0.15,hx=3.5,ly=0.75,cc=FALSE,pc=50,hpc=0.75,file="pre.stress",cca=FALSE){
  #Scale Seurat object & calculate # of PCs to use
  
  #Inputs:
  #sc10x = Seruat object
  #lx = x low threshold for hvg selection
  #hx = x high threshold for hvg selection
  #ly = y low threshold for hvg selection
  #cc = scale cell cycle?
  #genes.s = s phase genes
  #genes.g2m = g2m phase genes
  #pc = number of PCs to cacluate
  #hpc = max variance cutoff for PCs to use"
  #file = file for output
  #cca = cca experiment (if yes, don't re-scale)
  
  #Outputs:
  #result[1] = Seurat object
  #result[2] = hvg list
  #result[3] = # of PCs to use
  
  #Find variable genes
  sc10x <- FindVariableGenes(object=sc10x,mean.function=ExpMean,dispersion.function=LogVMR,x.low.cutoff=lx,x.high.cutoff=hx,y.cutoff=ly,do.plot=FALSE)
  genes.hvg <- rownames(sc10x@hvg.info[sc10x@hvg.info[,1]>=lx & sc10x@hvg.info[,1]<=hx & sc10x@hvg.info[,3]>=ly,])
  
  #Scale data
  if (cca==FALSE){
    gc()
    if (cc==TRUE){
      sc10x <- ScaleData(object=sc10x,vars.to.regress=c("nUMI","percent.mito","S.Score","G2M.Score"),display.progress=FALSE,do.par=TRUE,num.cores=45)
    } else {
      sc10x <- ScaleData(object=sc10x,vars.to.regress=c("nUMI","percent.mito"),display.progress=FALSE,do.par=TRUE,num.cores=45)
    }
    gc()
  }
  
  #Run PCA
  sc10x <- SetAllIdent(object=sc10x,id="ALL")
  sc10x <- RunPCA(object=sc10x,pc.genes=genes.hvg,do.print=FALSE,pcs.compute=pc)
  postscript(paste0("./analysis/qc/Plot_PCElbow_",file,".eps"))
  plot <- PCElbowPlot(object=sc10x,num.pc=pc)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20))
  plot(plot)
  dev.off()
  
  #Calculate PCs to use
  pc.use <- (sc10x@dr$pca@sdev)^2
  pc.use <- pc.use/sum(pc.use)
  pc.use <- cumsum(pc.use)
  pc.use <- min(which(pc.use>=hpc))
  
  results <- list(
    sc10x=sc10x,
    genes.hvg=genes.hvg,
    pc.use=pc.use
  )
  return(results)
}

scCluster <- function(sc10x,pc.use=10,res.use=0.1,folder="pre.stress",red="pca"){
  #Cluster
  
  #Inputs:
  #sc10x = Seruat object
  #pc.use = number of PCs to use
  #res.use = resolution to use
  #folder = folder for output
  
  #Outputs:
  #Seurat object
  
  #Calculate tSNE
  sc10x <- RunTSNE(object=sc10x,reduction.use=red,dims.use=1:pc.use,do.fast=TRUE)
  
  #Create tSNE grouped by samples
  postscript(paste0("./analysis/tSNE/",folder,"/tSNE_Sample.eps"))
  plot <- TSNEPlot(object=sc10x,group.by="samples",pt.size=2.5,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  
  #Cluster and save resolution
  gc()  
  sc10x <- FindClusters(object=sc10x,reduction.type=red,dims.use=1:pc.use,resolution=res.use,print.output=0,save.SNN=TRUE,force.recalc=TRUE)
  sc10x <- BuildClusterTree(sc10x,do.reorder=TRUE,reorder.numeric=TRUE,do.plot=FALSE)
  sc10x <- StashIdent(object=sc10x,save.name=paste0("res",res.use))
  
  #Create tSNE grouped by cluster
  postscript(paste0("./analysis/tSNE/",folder,"/tSNE_res",res.use,".eps"))
  plot <- TSNEPlot(object=sc10x,pt.size=5,do.label=TRUE,label.size=10,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  
  return(sc10x)
}

scStress <- function(sc10x,stg="go",res.use=1,pc.use=10,cut=0.95){
  #Runs custom PCA analysis for stress ID
  
  #Inputs:
  #sc10x = Seruat object
  #stg = geneset to use for stress ID
  #res.use = pre stress filter resulution
  #pc.use = PCs used pre stress filter

  #Outputs:
  #results[1] = Seurat object
  #results[2] = filtered no stress cell count
  #results[3] = Seurat object

  #Load stress genes
  if (stg=="go"){
    genes.stress <- read_csv("./genesets/DEG_C2.CGP.M10970.txt")
    genes.stress <- genes.stress[2:nrow(genes.stress),]
  } else if (stg=="dws"){
    genes.stress <- read_delim("./genesets/genes.deg.Stress.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
    genes.stress <- genes.stress[1]
    colnames(genes.stress) <- "scDWS.Stress"
  }
  genes.stress <- as.list(genes.stress)
  
  #Subset genes to only stress genes
  sc10x <- SetAllIdent(object=sc10x,id="ALL")
  sc10x.Stress <- as.data.frame(as.matrix(sc10x@data[rownames(sc10x@scale.data) %in% unlist(genes.stress),]))
  
  #Run PCA of subsetted stress genes
  sc10x.Stress <- t(sc10x.Stress)
  sc10x.Stress <- sc10x.Stress[,apply(sc10x.Stress,2,var)!=0]
  sc10x.Stress.pca <- prcomp(sc10x.Stress,center=TRUE,scale.=TRUE)
  sc10x.Stress.pca.pc1var <- round((sc10x.Stress.pca$sdev[1]^2)/sum(sc10x.Stress.pca$sdev^2)*100,0)
  sc10x.Stress.pca.pc2var <- round((sc10x.Stress.pca$sdev[2]^2)/sum(sc10x.Stress.pca$sdev^2)*100,0)
  sc10x.Stress.pca <- sc10x.Stress.pca$x[,1:2]
  colnames(x=sc10x.Stress.pca) <- paste0("Stress",1:2)
  if (skewness(sc10x.Stress.pca[,1])<0){
    sc10x.Stress.pca[,1] <- (-sc10x.Stress.pca[,1])
  }
  if (skewness(sc10x.Stress.pca[,2])<0){
    sc10x.Stress.pca[,2] <- (-sc10x.Stress.pca[,2])
  }
  sc10x <- SetDimReduction(object=sc10x,reduction.type="Stress",slot="cell.embeddings",new.data=sc10x.Stress.pca)
  sc10x <- SetDimReduction(object=sc10x,reduction.type="Stress",slot="key",new.data="Stress")
  
  #Generate PCA PC1 (Stress1/Stress-Score) and PC2 (Stress2)
  postscript("./analysis/pca/stress/PCA.eps")
  plot <- DimPlot(object=sc10x,reduction.use="Stress",pt.size=2.5,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+geom_density2d(color="black",bins=25,alpha=0.5)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot <- plot+xlab(paste0("Stress PC1 (",sc10x.Stress.pca.pc1var,"%)"))
  plot <- plot+ylab(paste0("Stress PC2 (",sc10x.Stress.pca.pc2var,"%)"))
  plot(plot)
  dev.off()
  
  #CDS
  cdf <- ecdf(GetCellEmbeddings(object=sc10x,reduction.type="Stress",dims.use=1))
  cut.x <- quantile(cdf,probs=cut)
  postscript("./analysis/pca/stress/CDF_Stress.eps")
  plot(cdf,main="Cumulative Distribution of Stress Score",xlab="Stress Score",ylab="CDF")
  abline(v=cut.x,col="red")
  dev.off()  
  
  #KDE of Stress1
  postscript("./analysis/pca/stress/Histo_Stress.eps")
  histo <- hist(GetCellEmbeddings(object=sc10x,reduction.type="Stress",dims.use=1),breaks=100,prob=TRUE,plot=TRUE,main="Distribution of Stress Score",xlab="Stress Score")
  d1 <- density(GetCellEmbeddings(object=sc10x,reduction.type="Stress",dims.use=1),n=100)
  lines(d1,col="blue")
  #ts_1 <- ts(d1$y)
  #tp1 <- turnpoints(ts_1)
  #pit1 <- min(d1$x[tp1$pits][d1$x[tp1$pits]>d1$x[tp1$peaks][d1$y[tp1$peaks]==max(d1$y[tp1$peaks])]])
  #pit1 <- min(d1$x[tp1$pits])
  abline(v=cut.x,col="red")
  dev.off()
  
  #Plot clusters
  ALL <- GetCellEmbeddings(object=sc10x,reduction.type="Stress",dims.use=1:2)
  Stress <- rownames(ALL[ALL[,1]>cut.x,])
  sc10x <- SetAllIdent(object=sc10x,id="ALL")
  sc10x <- SetIdent(object=sc10x,cells.use=Stress,ident.use="Stress")
  postscript("./analysis/pca/stress/PCA_Group.eps")
  plot <- DimPlot(object=sc10x,reduction.use="Stress",pt.size=2.5,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+geom_density2d(color="black",bins=25)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot <- plot+geom_vline(xintercept=cut.x,color="red",lwd=2.5)
  plot <- plot+xlab(paste0("Stress PC1 (",sc10x.Stress.pca.pc1var,"%)"))
  plot <- plot+ylab(paste0("Stress PC2 (",sc10x.Stress.pca.pc2var,"%)"))
  plot(plot)
  dev.off()
  
  #Subsample all cells (+Stress) to better visualize their clustering
  if (ncol(sc10x@data)<2500){
    rnd <- sample(1:ncol(sc10x@data),2500)
  } else {
    rnd <- 1:ncol(sc10x@data)
  }
  sc10x <- SetIdent(object=sc10x,cells.use=sc10x@data@Dimnames[[2]][rnd],ident.use="sample")
  sc10x <- SetIdent(object=sc10x,cells.use=Stress,ident.use="sample")
  sc10x.sample <- SubsetData(object=sc10x,ident.use="sample")
  sc10x.sample <- SetAllIdent(object=sc10x.sample,id="ALL")
  sc10x.sample <- SetIdent(object=sc10x.sample,cells.use=Stress,ident.use="Stress")
  sc10x.sample@data <- sc10x.sample@data[,c(colnames(sc10x.sample@data)[!colnames(sc10x.sample@data) %in% names(sc10x.sample@ident[sc10x.sample@ident=="Stress"])],names(sc10x.sample@ident[sc10x.sample@ident=="Stress"]))]

  #Generate tSNE plot of subsampled cells + Stress
  sc10x.sample@data <- sc10x.sample@data[,c(colnames(sc10x.sample@data)[!colnames(sc10x.sample@data) %in% Stress],Stress)]
  postscript("./analysis/tSNE/pre.stress/tSNE_Stress-SubSample.eps")
  plot <- TSNEPlot(object=sc10x.sample,pt.size=5,do.label=FALSE,label.size=15,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  sc10x <- SetIdent(object=sc10x,cells.use=Stress,ident.use="Stress")
  
  #Generate violin plot of stress gene expression
  anchors <- c("EGR1","FOS","JUN")
  for (i in anchors){
    if (i %in% rownames(sc10x@data)){
      postscript(paste0("./analysis/violin/stress/Violin_.",i,".eps"))
      plot <- VlnPlot(object=sc10x,features.plot=i,point.size.use=2.5,size.title.use=25,x.lab.rot=TRUE)
      plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20))
      plot(plot)
      dev.off()
  }}
  
  #save stress cell identity
  sc10x <- SetAllIdent(object=sc10x,id="ALL")
  sc10x <- SetIdent(object=sc10x,cells.use=Stress,ident.use="Stress")
  sc10x <- StashIdent(object=sc10x,save.name="ALL+Stress")
  
  #Remove stress pops and generate new samples tSNE
  sc10x <- SetAllIdent(object=sc10x,id=paste0("res",res.use))
  Number.Clusters <- length(unique(sc10x@ident))
  merge.sample <- round(prop.table(table(sc10x@meta.data$samples,sc10x@meta.data$'ALL+Stress'),1)*100,1)
  write.table(merge.sample,file="./analysis/table/Table_Stress.Samples.csv",row.names=TRUE,col.names=NA,append=FALSE,sep=",")
  merge.cluster <- round(prop.table(table(sc10x@ident,sc10x@meta.data$'ALL+Stress'),1)*100,1)
  write.table(merge.cluster,file="./analysis/table/Table_Stress.Clusters.csv",row.names=TRUE,col.names=NA,append=FALSE,sep=",")
  merge.cluster <- merge.cluster[,"Stress"]
  merge.cluster <- as.integer(names(merge.cluster[merge.cluster>50]))
  sc10x@ident <- plyr::mapvalues(x=sc10x@ident,from=merge.cluster,to=rep("Stress",length(merge.cluster)))
  sc10x@ident <- plyr::mapvalues(x=sc10x@ident,from=(1:Number.Clusters)[!(1:Number.Clusters %in% merge.cluster)],to=rep("ALL",length((1:Number.Clusters)[!(1:Number.Clusters %in% merge.cluster)])))
  sc10x <- SetIdent(object=sc10x,cells.use=Stress,ident.use="Stress")
  sc10x.subset <- SubsetData(object=sc10x,ident.use="ALL")
  sc10x.subset <- SetAllIdent(object=sc10x.subset,id=paste0("res",res.use))
  sc10x.subset <- BuildClusterTree(sc10x.subset,do.reorder=TRUE,reorder.numeric=TRUE,do.plot=FALSE)
  sc10x.subset <- StashIdent(object=sc10x.subset,save.name=paste0("res",res.use,"-Stress"))
  postscript(paste0("./analysis/tSNE/pre.stress/tSNE_","res",res.use,"-NoStress.eps"))
  plot <- TSNEPlot(object=sc10x.subset,pt.size=5,do.label=FALSE,label.size=15,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  counts.cell.filtered.stress <- length(sc10x.subset@data@Dimnames[[2]])
  
  results <- list(
    sc10x=sc10x.subset,
    counts.cell.filtered.stress=counts.cell.filtered.stress,
    sc10x.Stress <- sc10x
  )
  return(results)
}

scQuSAGE <- function(sc10x,gs,res.use=0.1,ds=25000,nm="Lin",folder="lin"){
  #Runs QuSAGE
  
  #Inputs:
  #sc10x = Seruat object
  #gs = geneset to use for correlation
  #res.use = calculated PC to use
  #nm = name of test
  #folder = folder for output
  
  #Outputs:
  #results[1] = Seurat object
  #results[2] = correlation table
  #results[3] = correlation results
  
  #Downsample and store expression matrix for QuSAGE
  sc10x <- SetAllIdent(object=sc10x,id=paste0("res",res.use))
  number.clusters <- length(unique(sc10x@ident))
  
  labels <- paste0("Cluster_",as.vector(factor(sc10x@ident)))
  
  cell.sample <- NULL
  for (i in 1:number.clusters){
    cell <- names(sc10x@ident[sc10x@ident==i])
    if (length(cell)>ds & ds!=0){
      rnd <- sample(1:length(cell),ds)
      cell <- cell[rnd]
    }
    cell.sample <- c(cell.sample,cell)
  }
  data <- sc10x@data[,colnames(sc10x@data) %in% cell.sample]
  data <- as.data.frame(as.matrix(data))
  labels <- labels[colnames(sc10x@data) %in% cell.sample]
  
  #Make labels for QuSAGE
  clust <- list()
  clust.comp <- list()
  for (i in 1:number.clusters){
    t <- labels
    t[!(t %in% paste0("Cluster_",i))] <- "REST"
    clust[i] <- list(i=t)
    rm(t)
    clust.comp[i] <- paste0("Cluster_",i,"-REST")
  }
  
  #Run QuSAGE
  for (i in 1:number.clusters){
    assign(paste0("results.",i),qusage(data,unlist(clust[i]),unlist(clust.comp[i]),gs))
  }
  
  #Generate ID table
  results.cor <- NULL
  results.cor <- qsTable(results.1)
  results.cor$Cluster <- 1
  for (i in 2:number.clusters){
    qs <- qsTable(get(paste0("results.",i)))
    qs$Cluster <- i
    results.cor <- rbind(results.cor,qs)
  }
  results.cor <- results.cor[,-3]
  rownames(results.cor) <- NULL
  results.clust.id <- NULL
  if (max(results.cor[results.cor[,4]==1,][,2])>=0){
    results.clust.id <- results.cor[results.cor[,4]==1,][which.max(results.cor[results.cor[,4]==1,][,2]),]
  } else {
    results.clust.id$pathway.name <- "Unknown"
    results.clust.id$log.fold.change <- 0
    results.clust.id$FDR <- 0
    results.clust.id$Cluster <- 1
    results.clust.id <- as.data.frame(results.clust.id)
  }
  for (i in 2:number.clusters){
    if (max(results.cor[results.cor[,4]==i,][,2])>=0){
      results.clust.id <- rbind(results.clust.id,results.cor[results.cor[,4]==i,][which.max(results.cor[results.cor[,4]==i,][,2]),])
    } else {
      results.clust.id <- rbind(results.clust.id,data.frame(pathway.name="Unknown",log.fold.change=0,FDR=0,Cluster=i))
    }
  }
  rownames(results.clust.id) <- NULL
  
  #Determine axes for correlation plots
  max.x.rg <- 0
  min.x.rg <- 0
  max.y.rg <- 0
  for (i in 1:number.clusters){
    qs <- get(paste0("results.",i))
    if (max(qs$path.mean)>max.x.rg){
      max.x.rg <- max(qs$path.mean)
    }
    if (min(qs$path.mean)<min.x.rg){
      min.x.rg <- min(qs$path.mean)
    }
    if (max(qs$path.PDF)>max.y.rg){
      max.y.rg <- max(qs$path.PDF)
  }}
  
  #Plot correlation plots by geneset
  for (i in 1:length(gs)){
    postscript(paste0("./analysis/cor/QuSAGE_",nm,".",names(gs)[i],".eps"))
    for (j in 1:number.clusters){
      qs <- get(paste0("results.",j))
      if (j==1){
        plotDensityCurves(qs,path.index=i,col=viridis(number.clusters)[j],main=names(gs)[i],xlim=c(min.x.rg-0.05,max.x.rg+0.05),ylim=c(0,50*ceiling(max.y.rg/50)),xlab="Gene Set Activation",lwd=5,cex.main=2.5,cex.axis=1.5,cex.lab=2)
      } else {
        plotDensityCurves(qs,path.index=i,add=TRUE,col=viridis(number.clusters)[j],lwd=5)
    }}
    leg <- paste0("Cluster ",1:number.clusters)
    legend("topright",legend=leg,lty=1,col=viridis(number.clusters),lwd=5,cex=2,ncol=1,bty="n",pt.cex=2)
    box(lwd=5)
    dev.off()
  }
  
  #Plot correlation plots by cluster
  for (i in 1:number.clusters){
    qs <- get(paste0("results.",i))
    postscript(paste0("./analysis/cor/QuSAGE_",nm,"_Clust",i,".eps"))
    for (j in 1:length(gs)){
        if (j==1){
          plotDensityCurves(qs,path.index=j,col=j,main=paste0("Cluster ",i),xlim=c(min.x.rg-0.05,max.x.rg+0.05),ylim=c(0,50*ceiling(max.y.rg/50)),xlab="Gene Set Activation",lwd=5,cex.main=2.5,cex.axis=1.5,cex.lab=2)
        } else {
          plotDensityCurves(qs,path.index=j,add=TRUE,col=j,lwd=5)
    }}
    legend("topright",legend=names(gs),lty=1,col=1:length(gs),lwd=5,cex=2,ncol=1,bty="n",pt.cex=2)
    box(lwd=5)
    dev.off()
  }
  
  #Save ident
  merge.cluster <- NULL
  for (i in 1:number.clusters){
    if (max(qsTable(get(paste0("results.",i)))[,2])>=0){
      merge.cluster <- c(merge.cluster,as.character(qsTable((get(paste0("results.",i))))[qsTable((get(paste0("results.",i))))[,2]==max(qsTable((get(paste0("results.",i))))[,2]) & qsTable((get(paste0("results.",i))))[,2]>0,1]))
    } else {
      merge.cluster <- c(merge.cluster,"Unknown")
    }}
  sc10x@ident <- plyr::mapvalues(x=sc10x@ident,from=1:number.clusters,to=merge.cluster)
  sc10x@ident <- factor(sc10x@ident,levels=c(names(gs),"Unknown"))
  sc10x <- StashIdent(object=sc10x,save.name=nm)
  
  postscript(paste0("./analysis/tSNE/",folder,"/tSNE_",nm,".eps"))
  plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  
  results <- list(
    sc10x=sc10x,
    results.cor=results.cor,
    results.clust.id=results.clust.id
  )
  names(results)=c("sc10x",paste0("results.cor.",nm),paste0("results.clust.",nm,".id"))
  return(results)
}


scQuSAGE.nocorplot <- function(sc10x,gs,res.use=0.1,ds=25000,nm="Lin",folder="lin"){
    #Runs QuSAGE
    
    #Inputs:
    #sc10x = Seruat object
    #gs = geneset to use for correlation
    #res.use = calculated PC to use
    #nm = name of test
    #folder = folder for output
    
    #Outputs:
    #results[1] = Seurat object
    #results[2] = correlation table
    #results[3] = correlation results
    
    #Downsample and store expression matrix for QuSAGE
    sc10x <- SetAllIdent(object=sc10x,id=paste0("res",res.use))
    number.clusters <- length(unique(sc10x@ident))
    
    labels <- paste0("Cluster_",as.vector(factor(sc10x@ident)))
    
    cell.sample <- NULL
    for (i in 1:number.clusters){
      cell <- names(sc10x@ident[sc10x@ident==i])
      if (length(cell)>ds & ds!=0){
        rnd <- sample(1:length(cell),ds)
        cell <- cell[rnd]
      }
      cell.sample <- c(cell.sample,cell)
    }
    data <- sc10x@data[,colnames(sc10x@data) %in% cell.sample]
    data <- as.data.frame(as.matrix(data))
    labels <- labels[colnames(sc10x@data) %in% cell.sample]
    
    #Make labels for QuSAGE
    clust <- list()
    clust.comp <- list()
    for (i in 1:number.clusters){
      t <- labels
      t[!(t %in% paste0("Cluster_",i))] <- "REST"
      clust[i] <- list(i=t)
      rm(t)
      clust.comp[i] <- paste0("Cluster_",i,"-REST")
    }
    
    #Run QuSAGE
    for (i in 1:number.clusters){
      assign(paste0("results.",i),qusage(data,unlist(clust[i]),unlist(clust.comp[i]),gs))
    }
    
  #Generate ID table
  results.cor <- NULL
  results.cor <- qsTable(results.1)
  results.cor$Cluster <- 1
  for (i in 2:number.clusters){
    qs <- qsTable(get(paste0("results.",i)))
    qs$Cluster <- i
    results.cor <- rbind(results.cor,qs)
  }
  results.cor <- results.cor[,-3]
  rownames(results.cor) <- NULL
  results.clust.id <- NULL
  if (max(results.cor[results.cor[,4]==1,][,2])>=0){
    results.clust.id <- results.cor[results.cor[,4]==1,][which.max(results.cor[results.cor[,4]==1,][,2]),]
  } else {
    results.clust.id$pathway.name <- "Unknown"
    results.clust.id$log.fold.change <- 0
    results.clust.id$FDR <- 0
    results.clust.id$Cluster <- 1
    results.clust.id <- as.data.frame(results.clust.id)
  }
  for (i in 2:number.clusters){
    if (max(results.cor[results.cor[,4]==i,][,2])>=0){
      results.clust.id <- rbind(results.clust.id,results.cor[results.cor[,4]==i,][which.max(results.cor[results.cor[,4]==i,][,2]),])
    } else {
      results.clust.id <- rbind(results.clust.id,data.frame(pathway.name="Unknown",log.fold.change=0,FDR=0,Cluster=i))
    }
  }
  rownames(results.clust.id) <- NULL
  
  #Save ident
  merge.cluster <- NULL
  for (i in 1:number.clusters){
    if (max(qsTable(get(paste0("results.",i)))[,2])>=0){
      merge.cluster <- c(merge.cluster,as.character(qsTable((get(paste0("results.",i))))[qsTable((get(paste0("results.",i))))[,2]==max(qsTable((get(paste0("results.",i))))[,2]) & qsTable((get(paste0("results.",i))))[,2]>0,1]))
    } else {
      merge.cluster <- c(merge.cluster,"Unknown")
    }}
  sc10x@ident <- plyr::mapvalues(x=sc10x@ident,from=1:number.clusters,to=merge.cluster)
  sc10x@ident <- factor(sc10x@ident,levels=c(unique(names(gs)),"Unknown"))
  sc10x <- StashIdent(object=sc10x,save.name=nm)
  
  postscript(paste0("./analysis/tSNE/",folder,"/tSNE_",nm,".eps"))
  plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  
  results <- list(
    sc10x=sc10x,
    results.cor=results.cor,
    results.clust.id=results.clust.id
  )
  names(results)=c("sc10x",paste0("results.cor.",nm),paste0("results.clust.",nm,".id"))
  return(results)
}


scQuSAGEsm <- function(sc10x,gs,ds=25000,nm="Lin",folder="lin"){
  #Runs QuSAGE
  
  #Inputs:
  #sc10x = Seruat object
  #gs = geneset to use for correlationtable
  #nm = name of test
  #folder = folder for output
  
  #Outputs:
  #correlation table
  
  #Downsample and store expression matrix for QuSAGE
  sc10x <- SetAllIdent(object=sc10x,id=nm)
  clusters <- unique(sc10x@ident)
  
  labels <- as.vector(factor(sc10x@ident))

  cell.sample <- NULL
  for (i in clusters){
    cell <- names(sc10x@ident[sc10x@ident==i])
    if (length(cell)>ds & ds!=0){
      rnd <- sample(1:length(cell),ds)
      cell <- cell[rnd]
    }
    cell.sample <- c(cell.sample,cell)
  }
  data <- sc10x@data[,colnames(sc10x@data) %in% cell.sample]
  data <- as.data.frame(as.matrix(data))
  labels <- labels[colnames(sc10x@data) %in% cell.sample]
  
  
  #Make labels for QuSAGE
  clust <- list()
  clust.comp <- list()
  for (i in clusters){
    t <- labels
    t[!(t == i)] <- "REST"
    clust[i] <- list(i=t)
    rm(t)
    clust.comp[i] <- paste0(i,"-REST")
  }
  
  #Run QuSAGE
  for (i in clusters){
    assign(paste0("results.",i),qusage(data,unlist(clust[i]),unlist(clust.comp[i]),gs))
  }
  
  #Generate ID table
  results.cor <- NULL
  results.cor <- qsTable(get(paste0("results.",clusters[1])),number=length(gs))
  results.cor$Cluster <- clusters[1]
  for (i in clusters[-1]){
    qs <- qsTable(get(paste0("results.",i)),number=length(gs))
    qs$Cluster <- i
    results.cor <- rbind(results.cor,qs)
  }
  results.cor <- results.cor[,-3]
  rownames(results.cor) <- NULL
  
  #Determine axes for correlation plots
  max.x.rg <- 0
  min.x.rg <- 0
  max.y.rg <- 0
  for (i in clusters){
    qs <- get(paste0("results.",i))
    if (max(qs$path.mean)>max.x.rg){
      max.x.rg <- max(qs$path.mean)
    }
    if (min(qs$path.mean)<min.x.rg){
      min.x.rg <- min(qs$path.mean)
    }
    if (max(qs$path.PDF)>max.y.rg){
      max.y.rg <- max(qs$path.PDF)
    }}
  
  #Plot correlation plots by geneset
  for (i in 1:length(gs)){
    postscript(paste0("./analysis/cor/QuSAGE_",nm,".",folder,".",names(gs)[i],".eps"))
    for (j in clusters){
      qs <- get(paste0("results.",j))
      if (j==clusters[1]){
        plotDensityCurves(qs,path.index=i,col=which(clusters == j),main=names(gs)[i],xlim=c(min.x.rg-0.05,max.x.rg+0.05),ylim=c(0,50*ceiling(max.y.rg/50)),xlab="Gene Set Activation",lwd=5,cex.main=2.5,cex.axis=1.5,cex.lab=2)
      } else {
        plotDensityCurves(qs,path.index=i,add=TRUE,col=which(clusters == j),lwd=5)
      }}
    leg <- clusters
    legend("topright",legend=leg,lty=1,col=1:length(clusters),lwd=5,cex=2,ncol=1,bty="n",pt.cex=2)
    
    dev.off()
  }
  
  #Plot correlation plots by cluster
  for (i in clusters){
    qs <- get(paste0("results.",i))
    postscript(paste0("./analysis/cor/QuSAGE_",nm,".",folder,"_",i,".eps"))
    for (j in 1:length(gs)){
      if (j==1){
        plotDensityCurves(qs,path.index=j,col=j,main=i,xlim=c(min.x.rg-0.05,max.x.rg+0.05),ylim=c(0,50*ceiling(max.y.rg/50)),xlab="Gene Set Activation",lwd=5,cex.main=2.5,cex.axis=1.5,cex.lab=2)
      } else {
        plotDensityCurves(qs,path.index=j,add=TRUE,col=j,lwd=5)
      }}
    legend("topright",legend=names(gs),lty=1,col=1:length(gs),lwd=5,cex=2,ncol=1,bty="n",pt.cex=2)
    
    dev.off()
  }
    
  
  return(results.cor)
}



scQuSAGElg <- function(sc10x,gs,ds=25000,nm="Lin",folder="lin"){
  #Runs QuSAGE
  
  #Inputs:
  #sc10x = Seruat object
  #gs = geneset to use for correlationtable
  #nm = name of test
  #folder = folder for output
  
  #Outputs:
  #correlation table
  
  #Downsample and store expression matrix for QuSAGE
  sc10x <- SetAllIdent(object=sc10x,id=nm)
  clusters <- unique(sc10x@ident)
  
  labels <- as.vector(factor(sc10x@ident))
  
  cell.sample <- NULL
  for (i in clusters){
    cell <- names(sc10x@ident[sc10x@ident==i])
    if (length(cell)>ds & ds!=0){
      rnd <- sample(1:length(cell),ds)
      cell <- cell[rnd]
    }
    cell.sample <- c(cell.sample,cell)
  }
  data <- sc10x@data[,colnames(sc10x@data) %in% cell.sample]
  data <- as.data.frame(as.matrix(data))
  labels <- labels[colnames(sc10x@data) %in% cell.sample]
  
  #Make labels for QuSAGE
  clust <- list()
  clust.comp <- list()
  for (i in clusters){
    t <- labels
    t[!(t == i)] <- "REST"
    clust[i] <- list(i=t)
    rm(t)
    clust.comp[i] <- paste0(i,"-REST")
  }
  
  #Run QuSAGE
  for (i in clusters){
    assign(paste0("results.",i),qusage(data,unlist(clust[i]),unlist(clust.comp[i]),gs))
  }
  
  #Generate ID table
  results.cor <- NULL
  results.cor <- qsTable(get(paste0("results.",clusters[1])),number=length(gs))
  results.cor$Cluster <- clusters[1]
  for (i in clusters[-1]){
    qs <- qsTable(get(paste0("results.",i)),number=length(gs))
    qs$Cluster <- i
    results.cor <- rbind(results.cor,qs)
  }
  results.cor <- results.cor[,-3]
  rownames(results.cor) <- NULL
    
  #Plot correlation plot
  for (i in clusters){
    qs <- get(paste0("results.",i))
    postscript(paste0("./analysis/cor/QuSAGE_",i,".",folder,".eps"))
    plotCIs(qs,path.index=1:numPathways(qs))
    dev.off()
    tab <- qsTable(qs,number=numPathways(qs),sort.by="logFC")
    tab <- tab[tab$FDR<0.05,]
    tab <- tab[tab$log.fold.change>0,]
    tab <- tab[order(-tab[,2]),]
    write.table(tab,file=paste0("./analysis/cor/QuSAGE_",i,".",folder,".Table.csv"),row.names=TRUE,col.names=NA,append=FALSE,sep=",")
  }


  return(results.cor)
}


scNE <- function(sc10x,neg="EurUro",cut=0.95){
  #Runs custom PCA analysis for NE ID
  
  #Inputs:
  #sc10x = Seruat object
  #neg = geneset to use for NE ID
  
  #Outputs:
  #Seurat object
  
  #Load NE genes
  if (neg=="EurUro"){
    genes.ne <- read_delim("./genesets/EurUrol.2005.NE.txt","\t",escape_double=FALSE,trim_ws=TRUE,col_names=FALSE)
  } else if (neg=="dws"){
    genes.ne <- read_delim("./genesets/genes.deg.NE.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
    genes.ne <- genes.ne[1]
    colnames(genes.ne) <- "scDWS.ne"
  }
  genes.ne <- as.list(genes.ne[1])
  
  #Subset genes to only NE genes
  sc10x <- SetAllIdent(object=sc10x,id="ALL")
  sc10x.NE <- as.data.frame(as.matrix(sc10x@data[rownames(sc10x@scale.data) %in% unlist(genes.ne),]))
  
  #Run PCA of subsetted NE genes
  sc10x.NE <- t(sc10x.NE)
  sc10x.NE <- sc10x.NE[,apply(sc10x.NE,2,var)!=0]
  sc10x.NE.pca <- prcomp(sc10x.NE,center=TRUE,scale.=TRUE)
  sc10x.NE.pca.pc1var <- round((sc10x.NE.pca$sdev[1]^2)/sum(sc10x.NE.pca$sdev^2)*100,0)
  sc10x.NE.pca.pc2var <- round((sc10x.NE.pca$sdev[2]^2)/sum(sc10x.NE.pca$sdev^2)*100,0)
  sc10x.NE.pca <- sc10x.NE.pca$x[,1:2]
  colnames(x=sc10x.NE.pca) <- paste0("NE",1:2)
  if (skewness(sc10x.NE.pca[,1])<0){
    sc10x.NE.pca[,1] <- (-sc10x.NE.pca[,1])
  }
  if (skewness(sc10x.NE.pca[,2])<0){
    sc10x.NE.pca[,2] <- (-sc10x.NE.pca[,2])
  }
  sc10x <- SetDimReduction(object=sc10x,reduction.type="NE",slot="cell.embeddings",new.data=sc10x.NE.pca)
  sc10x <- SetDimReduction(object=sc10x,reduction.type="NE",slot="key",new.data="NE")
  
  #Generate PCA PC1 (NE1/NE-Score) and PC2 (NE2)
  postscript("./analysis/pca/ne/PCA.eps")
  plot <- DimPlot(object=sc10x,reduction.use="NE",pt.size=2.5,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot <- plot+xlab(paste0("NE PC1 (",sc10x.NE.pca.pc1var,"%)"))
  plot <- plot+ylab(paste0("NE PC2 (",sc10x.NE.pca.pc2var,"%)"))
  plot(plot)
  dev.off()
  
  #CDF
  cdf <- ecdf(GetCellEmbeddings(object=sc10x,reduction.type="NE",dims.use=1))
  cut.x <- quantile(cdf,probs=cut)
  postscript("./analysis/pca/ne/CDF_NE.eps")
  plot(cdf,main="Cumulative Distribution of NE Score",xlab="NE Score",ylab="CDF")
  abline(v=cut.x,col="red")
  dev.off()
  
  #Find first turning point on a KDE of NE1
  postscript("./analysis/pca/ne/Histo_NE.eps")
  histo <- hist(GetCellEmbeddings(object=sc10x,reduction.type="NE",dims.use=1),breaks=100,probability=TRUE,plot=TRUE)
  d1 <- density(GetCellEmbeddings(object=sc10x,reduction.type="NE",dims.use=1),n=100)
  lines(d1,col="blue")
  #ts_1 <- ts(d1$y)
  #tp1 <- turnpoints(ts_1)
  #pit1 <- min(d1$x[tp1$pits][d1$x[tp1$pits]>d1$x[tp1$peaks][d1$y[tp1$peaks]==max(d1$y[tp1$peaks])]])
  #pit1 <- min(d1$x[tp1$pits])
  abline(v=cut.x,col="red")
  dev.off()
  
  #Plot clusters
  ALL <- GetCellEmbeddings(object=sc10x,reduction.type="NE",dims.use=1:2)
  NE <- rownames(ALL[ALL[,1]>cut.x,])
  sc10x <- SetAllIdent(object=sc10x,id="ALL")
  sc10x <- SetIdent(object=sc10x,cells.use=NE,ident.use="NE")
  postscript("./analysis/pca/ne/PCA_Group.eps")
  plot <- DimPlot(object=sc10x,reduction.use="NE",pt.size=2.5,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot <- plot+geom_vline(xintercept=cut.x,color="red",lwd=2.5)
  plot <- plot+xlab(paste0("NE PC1 (",sc10x.NE.pca.pc1var,"%)"))
  plot <- plot+ylab(paste0("NE PC2 (",sc10x.NE.pca.pc2var,"%)"))
  plot(plot)
  dev.off()
  
  #Subsample all cells (+NE) to better visualize their clustering
  if (ncol(sc10x@data)<2500){
    rnd <- sample(1:ncol(sc10x@data),2500)
  } else {
    rnd <- 1:ncol(sc10x@data)
  }
  sc10x <- SetIdent(object=sc10x,cells.use=sc10x@data@Dimnames[[2]][rnd],ident.use="sample")
  sc10x <- SetIdent(object=sc10x,cells.use=NE,ident.use="sample")
  if (ncol(sc10x@data)>2500){
    sc10x.sample <- SubsetData(object=sc10x,ident.use="sample")
  } else {
    sc10x.sample <-  sc10x
  }
  sc10x.sample <- SetAllIdent(object=sc10x.sample,id="ALL")
  sc10x.sample <- SetIdent(object=sc10x.sample,cells.use=NE,ident.use="NE")
  sc10x.sample@data <- sc10x.sample@data[,c(colnames(sc10x.sample@data)[!colnames(sc10x.sample@data) %in% names(sc10x.sample@ident[sc10x.sample@ident=="NE"])],names(sc10x.sample@ident[sc10x.sample@ident=="NE"]))]
  
  
  #Generate tSNE plot of subsampled cells + NE
  sc10x.sample@data <- sc10x.sample@data[,c(colnames(sc10x.sample@data)[!colnames(sc10x.sample@data) %in% names(sc10x.sample@ident[sc10x.sample@ident=="NE"])],names(sc10x.sample@ident[sc10x.sample@ident=="NE"]))]
  postscript("./analysis/tSNE/ne/tSNE_SubSample.eps")
  plot <- TSNEPlot(object=sc10x.sample,pt.size=5,do.label=FALSE,label.size=15,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  sc10x <- SetIdent(object=sc10x,cells.use=NE,ident.use="NE")
  
  #Generate violin plot of stress gene expression
  anchors <- c("CHGA","CHGB","SYP")
  for (i in anchors){
    if (i %in% rownames(sc10x@data)){
      postscript(paste0("./analysis/violin/ne/Violin_.",i,".eps"))
      plot <- VlnPlot(object=sc10x,features.plot=i,point.size.use=2.5,size.title.use=25,x.lab.rot=TRUE)
      plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20))
      plot(plot)
      dev.off()
  }}
  
  #save stress cell identity
  sc10x <- SetAllIdent(object=sc10x,id="ALL")
  sc10x <- SetIdent(object=sc10x,cells.use=NE,ident.use="NE")
  sc10x <- StashIdent(object=sc10x,save.name="NE")
  
  return(sc10x)
}


scMergeSubClust <- function(sc10x,i="ALL",g="ALL",nm="MergeSubClust"){
  #Merge subclusters
  
  #Inputs:
  #sc10x = Seruat object
  #i = identity to use
  #g = group to merge
  #nm = merge identity name
  
  #Outputs:
  #Seurat object
  
  sc10x <- SetAllIdent(object=sc10x,id=i)
  for (i in 1:length(g)){
    cell <- names(sc10x@ident[grep(paste0("^",g[i]),as.character(sc10x@ident))])
    sc10x <- SetIdent(object=sc10x,cells.use=cell,ident.use=g[i])
  }
  sc10x <- StashIdent(object=sc10x,save.name=nm)
  
  return(sc10x)
}


scMerge <- function(sc10x,sc10x.1,sc10x.2,i.1,i.2,nm="Merge"){
  #Merge ID's
  
  #Inputs:
  #sc10x = Seruat object
  #sc10x.1 = Seruat object
  #sc10x.2 = Seruat object
  #i.1 = identity to use
  #i.2 = identity to use
  #nm = merge identity name
  
  #Outputs:
  #Seurat object
  
  sc10x.1 <- SetAllIdent(object=sc10x.1,id=i.1)
  cell.1 <- names(sc10x.1@ident[sc10x.1@ident!="ALL"])
  sc10x.2 <- SetAllIdent(object=sc10x.2,id=i.2)
  cell.2 <- names(sc10x.2@ident[sc10x.2@ident!="ALL"])
  
  sc10x <- SetIdent(object=sc10x,cells.use=cell.1,ident.use=as.character(factor(sc10x.1@ident[sc10x.1@ident!="ALL"])))
  sc10x <- SetIdent(object=sc10x,cells.use=cell.2,ident.use=as.character(factor(sc10x.2@ident[sc10x.2@ident!="ALL"])))
  sc10x@ident <- factor(sc10x@ident,levels=c(unique(as.character(factor(sc10x.1@ident))),unique(as.character(factor(sc10x.2@ident)))))
  sc10x <- StashIdent(object=sc10x,save.name=nm)
  
  postscript(paste0("./analysis/tSNE/merge/tSNE_",nm,".eps"))
  plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.return=TRUE,vector.friendly=FALSE)
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
  
  return(sc10x)
}


scTables <- function(sc10x,i.1,i.2){
  write.table(table(sc10x@meta.data[,i.1],sc10x@meta.data[,i.2]),file=paste0("./analysis/table/Table_",i.1,"_",i.2,".csv"),row.names=TRUE,col.names=NA,append=FALSE,sep=",")
  write.table(round(prop.table(table(sc10x@meta.data[,i.1],sc10x@meta.data[,i.2]),1)*100,1),file=paste0("./analysis/table/ProbTable_",i.1,"_",i.2,".csv"),row.names=TRUE,col.names=NA,append=FALSE,sep=",")
}


scDEG <- function(sc10x,i="Lin",g.1="Epi",g.2="St",pct=0.25,t=2){
  sc10x <- SetAllIdent(object=sc10x,id=i)
  genes.deg <- FindMarkers(object=sc10x,ident.1=g.1,ident.2=g.2,min.pct=pct,only.pos=TRUE,thresh.use=0.25,logfc.threshold=log(t))
  genes.deg <- genes.deg[genes.deg$p_val_adj<=0.05,]
  
  return(genes.deg)
}



scCCA <-  function(sc10x.1,sc10x.2,nm.1="D17",nm.2="D27",cc=FALSE){
  gc()
  if (cc==TRUE){
    sc10x.1 <- ScaleData(object=sc10x.1,vars.to.regress=c("nUMI","percent.mito","S.Score","G2M.Score"),display.progress=FALSE,do.par=TRUE,num.cores=45)
  } else {
    sc10x.1 <- ScaleData(object=sc10x.1,vars.to.regress=c("nUMI","percent.mito"),display.progress=FALSE,do.par=TRUE,num.cores=45)
  }
  gc()
  
  gc()
  if (cc==TRUE){
    sc10x.2 <- ScaleData(object=sc10x.2,vars.to.regress=c("nUMI","percent.mito","S.Score","G2M.Score"),display.progress=FALSE,do.par=TRUE,num.cores=45)
  } else {
    sc10x.2 <- ScaleData(object=sc10x.2,vars.to.regress=c("nUMI","percent.mito"),display.progress=FALSE,do.par=TRUE,num.cores=45)
  }
  gc()
  
  sc10x.1 <- FindVariableGenes(sc10x.1,do.plot=FALSE)
  sc10x.2 <- FindVariableGenes(sc10x.2,do.plot=FALSE)
  genes.hvg.1 <- head(rownames(sc10x.1@hvg.info),1000)
  genes.hvg.2 <- head(rownames(sc10x.2@hvg.info),1000)
  genes.hvg.Comb <- unique(c(genes.hvg.1,genes.hvg.2))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.D17@scale.data))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.D27@scale.data))
  
  sc10x.1@meta.data$patient <- nm.1
  sc10x.2@meta.data$patient <- nm.2
  sc10x <- RunCCA(sc10x.1,sc10x.2,genes.use=genes.hvg.Comb,num.cc=25,add.cell.id1=nm.1,add.cell.id2=nm.2)
  rm(sc10x.1)
  rm(sc10x.2)
  
  postscript("./analysis/cca/CCA.eps",paper="special",width=10,height=5,horizontal=FALSE)
  DimPlot(sc10x,reduction.use="cca",group.by="patient",do.return=FALSE,pt.size=0.05)
  dev.off()
  postscript("./analysis/cca/Violin.CCA1.Raw.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="CC1",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  postscript("./analysis/cca/Violin.CCA2.Raw.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="CC2",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  #postscript("./analysis/cca/Bicor.eps",paper="special",width=10,height=5,horizontal=FALSE)
  #MetageneBicorPlot(sc10x,dims.eval=1:50,grouping.var="patient")
  #dev.off()
  
  gc()
  sc10x <- AlignSubspace(sc10x,reduction.type="cca",grouping.var="patient",dims.align=1:15)
  gc()
  
  postscript("./analysis/cca/CCA.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  DimPlot(sc10x,reduction.use="cca.aligned",group.by="patient",do.return=FALSE,pt.size=0.05)
  dev.off()
  postscript("./analysis/cca/Violin.CCA1.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="ACC1",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  postscript("./analysis/cca/Violin.CCA2.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="ACC2",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  
  results <- list(
    sc10x=sc10x,
    genes.hvg.cca=genes.hvg.Comb
  )
  return(results)
}


sc3CCA <-  function(sc10x.1,sc10x.2,sc10x.3,nm.1="D17",nm.2="D27",nm.3="D35",cc=FALSE,cca=25,acca=15,lx=0.15,hx=3.5,ly=0.75){
  sc10x.1 <- FindVariableGenes(object=sc10x.1,mean.function=ExpMean,dispersion.function=LogVMR,x.low.cutoff=lx,x.high.cutoff=hx,y.cutoff=ly,do.plot=FALSE)
  sc10x.2 <- FindVariableGenes(object=sc10x.2,mean.function=ExpMean,dispersion.function=LogVMR,x.low.cutoff=lx,x.high.cutoff=hx,y.cutoff=ly,do.plot=FALSE)
  sc10x.3 <- FindVariableGenes(object=sc10x.3,mean.function=ExpMean,dispersion.function=LogVMR,x.low.cutoff=lx,x.high.cutoff=hx,y.cutoff=ly,do.plot=FALSE)
  genes.hvg.1 <- sc10x.1@var.genes
  genes.hvg.2 <- sc10x.2@var.genes
  genes.hvg.3 <- sc10x.3@var.genes
  genes.hvg.Comb <- unique(c(genes.hvg.1,genes.hvg.2,genes.hvg.3))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.1@data))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.2@data))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.3@data))
  
  sc10x.1@meta.data$patient <- nm.1
  sc10x.2@meta.data$patient <- nm.2
  sc10x.3@meta.data$patient <- nm.3
  sc10x.1 <- RenameCells(sc10x.1,nm.1)
  sc10x.2 <- RenameCells(sc10x.2,nm.2)
  sc10x.3 <- RenameCells(sc10x.3,nm.3)
  sc10x.list <- list(sc10x.1,sc10x.2,sc10x.3)
  sc10x <- RunMultiCCA(sc10x.list,genes.use=genes.hvg.Comb,num.cc=cca)
  rm(sc10x.1)
  rm(sc10x.2)
  rm(sc10x.3)
  rm(sc10x.list)
  
  postscript("./analysis/cca/CCA.eps",paper="special",width=10,height=5,horizontal=FALSE)
  DimPlot(sc10x,reduction.use="cca",group.by="patient",do.return=FALSE,pt.size=0.05)
  dev.off()
  postscript("./analysis/cca/Violin.CCA1.Raw.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="CC1",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  postscript("./analysis/cca/Violin.CCA2.Raw.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="CC2",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  postscript("./analysis/cca/Bicor.eps",paper="special",width=10,height=5,horizontal=FALSE)
  MetageneBicorPlot(sc10x,dims.eval=1:cca,grouping.var="patient")
  dev.off()
  
  gc()
  sc10x <- AlignSubspace(sc10x,reduction.type="cca",grouping.var="patient",dims.align=1:acca)
  gc()
  
  postscript("./analysis/cca/CCA.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  DimPlot(sc10x,reduction.use="cca.aligned",group.by="patient",do.return=FALSE,pt.size=0.05)
  dev.off()
  postscript("./analysis/cca/Violin.CCA1.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="ACC1",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  postscript("./analysis/cca/Violin.CCA2.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="ACC2",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  
  results <- list(
    sc10x=sc10x,
    genes.hvg.cca=genes.hvg.Comb
  )
  return(results)
}


sc4CCA <-  function(sc10x.1,sc10x.2,sc10x.3,sc10x.4,nm.1="D17",nm.2="D27",nm.3="D35",nm.4,cc=FALSE,cca=25,acca=15,lx=0.15,hx=3.5,ly=0.75){
  sc10x.1 <- FindVariableGenes(object=sc10x.1,mean.function=ExpMean,dispersion.function=LogVMR,x.low.cutoff=lx,x.high.cutoff=hx,y.cutoff=ly,do.plot=FALSE)
  sc10x.2 <- FindVariableGenes(object=sc10x.2,mean.function=ExpMean,dispersion.function=LogVMR,x.low.cutoff=lx,x.high.cutoff=hx,y.cutoff=ly,do.plot=FALSE)
  sc10x.3 <- FindVariableGenes(object=sc10x.3,mean.function=ExpMean,dispersion.function=LogVMR,x.low.cutoff=lx,x.high.cutoff=hx,y.cutoff=ly,do.plot=FALSE)
  sc10x.4 <- FindVariableGenes(object=sc10x.4,mean.function=ExpMean,dispersion.function=LogVMR,x.low.cutoff=lx,x.high.cutoff=hx,y.cutoff=ly,do.plot=FALSE)
  
  genes.hvg.1 <- sc10x.1@var.genes
  genes.hvg.2 <- sc10x.2@var.genes
  genes.hvg.3 <- sc10x.3@var.genes
  genes.hvg.4 <- sc10x.4@var.genes
  genes.hvg.Comb <- unique(c(genes.hvg.1,genes.hvg.2,genes.hvg.3,genes.hvg.4))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.1@data))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.2@data))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.3@data))
  genes.hvg.Comb <- intersect(genes.hvg.Comb,rownames(sc10x.4@data))
  
  sc10x.1@meta.data$patient <- nm.1
  sc10x.2@meta.data$patient <- nm.2
  sc10x.3@meta.data$patient <- nm.3
  sc10x.4@meta.data$patient <- nm.4
  sc10x.1 <- RenameCells(sc10x.1,nm.1)
  sc10x.2 <- RenameCells(sc10x.2,nm.2)
  sc10x.3 <- RenameCells(sc10x.3,nm.3)
  sc10x.4 <- RenameCells(sc10x.4,nm.4)
  sc10x.list <- list(sc10x.1,sc10x.2,sc10x.3,sc10x.4)
  sc10x <- RunMultiCCA(sc10x.list,genes.use=genes.hvg.Comb,num.cc=cca)
  rm(sc10x.1)
  rm(sc10x.2)
  rm(sc10x.3)
  rm(sc10x.4)
  rm(sc10x.list)
  
  postscript("./analysis/cca/CCA.eps",paper="special",width=10,height=5,horizontal=FALSE)
  DimPlot(sc10x,reduction.use="cca",group.by="patient",do.return=FALSE,pt.size=0.05)
  dev.off()
  postscript("./analysis/cca/Violin.CCA1.Raw.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="CC1",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  postscript("./analysis/cca/Violin.CCA2.Raw.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="CC2",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  postscript("./analysis/cca/Bicor.eps",paper="special",width=10,height=5,horizontal=FALSE)
  MetageneBicorPlot(sc10x,dims.eval=1:cca,grouping.var="patient")
  dev.off()
  
  gc()
  sc10x <- AlignSubspace(sc10x,reduction.type="cca",grouping.var="patient",dims.align=1:acca)
  gc()
  
  postscript("./analysis/cca/CCA.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  DimPlot(sc10x,reduction.use="cca.aligned",group.by="patient",do.return=FALSE,pt.size=0.05)
  dev.off()
  postscript("./analysis/cca/Violin.CCA1.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="ACC1",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  postscript("./analysis/cca/Violin.CCA2.Aligned.eps",paper="special",width=10,height=5,horizontal=FALSE)
  plot <- VlnPlot(sc10x,features.plot="ACC2",group.by="patient",size.title.use=20,point.size.use=0.05)
  plot(plot)  
  dev.off()
  
  results <- list(
    sc10x=sc10x,
    genes.hvg.cca=genes.hvg.Comb
  )
  return(results)
}


sctSNECustCol <- function(sc10x,i,bl,rd,file){
  sc10x <- SetAllIdent(object=sc10x,id=i)
  if (any(sc10x@ident == "Unknown")){
    sc10x@ident <- factor(sc10x@ident,levels=c(bl,rd))
  } else {
    sc10x@ident <- factor(sc10x@ident,levels=c(bl,rd,"Unknown"))
  }
  postscript(paste0("./analysis/diy/tSNE_",file,".",i,".CustCol.eps"))
  if (length(bl)==1 & length(rd)==1){
    if (length(sc10x@ident[sc10x@ident=="Unknown"])>0){
      plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=c(brewer.pal(5,"Blues")[5],brewer.pal(5,"Reds")[5],"grey50"))
    } else {
      plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=c(brewer.pal(5,"Blues")[5],brewer.pal(5,"Reds")[5]))
    }
    plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
    plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
    plot(plot)
  } else {
    if (bl!=""){
      plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=c(brewer.pal((length(bl)+1),"Blues")[2:(length(bl)+1)],brewer.pal((length(rd)+1),"Reds")[2:(length(rd)+1)]))
    } else {
      plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=brewer.pal((length(rd)+1),"Reds")[2:(length(rd)+1)])
    }
    plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
    plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
    plot(plot)
  }
  dev.off()
}

sctSNE3CustCol <- function(sc10x,i,bl,rd,gn,file){
  if (gn!=""){
    gn.col <- NULL
    if (length(sc10x@ident[sc10x@ident=="B-cells"])>0){
      gn.col <- c(gn.col,"#BAE4B3")
    }
    if (length(sc10x@ident[sc10x@ident=="T-cells"])>0){
      gn.col <- c(gn.col,"#74C476")
    }
    if (length(sc10x@ident[sc10x@ident=="Macrophages"])>0){
      gn.col <- c(gn.col,"#31A354")
    }
    if (length(sc10x@ident[sc10x@ident=="Mast cells"])>0){
      gn.col <- c(gn.col,"#006D2C")
    }
  }
  sc10x <- SetAllIdent(object=sc10x,id=i)
  if (any(sc10x@ident == "Unknown")){
    sc10x@ident <- factor(sc10x@ident,levels=c(bl,rd,gn))
  } else {
    sc10x@ident <- factor(sc10x@ident,levels=c(bl,rd,gn,"Unknown"))
  }
  postscript(paste0("./analysis/diy/tSNE_",file,".",i,".CustCol.eps"))
  if (length(bl)==1 & length(rd)==1 & length(gn)==1){
    if (length(sc10x@ident[sc10x@ident=="Unknown"])>0){
      plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=c(brewer.pal(5,"Blues")[5],brewer.pal(5,"Reds")[5],brewer.pal(5,"Greens")[5],"grey50"))
    } else {
      plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=c(brewer.pal(5,"Blues")[5],brewer.pal(5,"Reds")[5],brewer.pal(5,"Greens")[5]))
    }
    plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
    plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
    plot(plot)
  } else {
    if (bl!=""){
      plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=c(brewer.pal((length(bl)+1),"Blues")[2:(length(bl)+1)],brewer.pal((length(rd)+1),"Reds")[2:(length(rd)+1)],gn.col))
    } else {
      plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=brewer.pal((length(rd)+1),"Reds")[2:(length(rd)+1)])
    }
    plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
    plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
    plot(plot)
  }
  dev.off()
}

sctSNEbwCol <- function(sc10x,i,file,files){
  sc10x <- SetAllIdent(object=sc10x,id=i)
  sc10x@ident <- factor(sc10x@ident,levels=c("Epi","BE","LE","OE1","OE_SCGB","OE2","OE_KRT13","St","Fib","SM","Endo","Leu","Unknown",1:50))
  postscript(paste0("./analysis/diy/tSNE_",files,".",file,"_",i,".bwCol.eps"))
  plot <- TSNEPlot(object=sc10x,pt.size=2.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=colorRampPalette(c("white","black"))(length(unique(sc10x@ident))+2)[3:(length(unique(sc10x@ident))+2)])
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
}


sctSNEHighlight <- function(sc10x,i,g,file=""){
  sc10x <- SetAllIdent(object=sc10x,id=i)
  sc10x <- SetIdent(object=sc10x,cells.use=names(sc10x@ident[!(sc10x@ident %in% g)]),ident.use="Background")
  sc10x@ident <- factor(sc10x@ident,levels=c(g,"Background"))
  sc10x@data <- sc10x@data[,c(colnames(sc10x@data)[!colnames(sc10x@data) %in% names(sc10x@ident[sc10x@ident==g])],names(sc10x@ident[sc10x@ident==g]))]
  
  postscript(paste0("./analysis/diy/tSNE_",file,".",i,"_",g,".Highlight.eps"))
  plot <- TSNEPlot(object=sc10x,pt.size=0.5,do.label=FALSE,label.size=10,do.return=TRUE,vector.friendly=FALSE,colors.use=c("darkred","grey"))
  plot <- plot+theme(axis.text.x=element_text(size=20),axis.text.y=element_text(size=20),axis.title.x=element_text(size=20),axis.title.y=element_text(size=20),legend.text=element_text(size=20))
  plot <- plot+guides(colour=guide_legend(override.aes=list(size=10)))
  plot(plot)
  dev.off()
}

scCustHeatmap <- function(sc10x,i,gs,g){
  sc10x <- SetAllIdent(object=sc10x,id=i)
  postscript(paste0("./analysis/diy/Heatmap.",i,".Cust.eps"),paper="special",width=10,height=5,horizontal=FALSE)
  plot(DoHeatmap(sc10x,genes.use=gs,slim.col.label=TRUE,group.label.rot=TRUE,group.spacing=0.25,cex.row=2.5,group.order=g))
  dev.off()
  tryCatch({
    png(paste0("./analysis/diy/Heatmap.",i,".Cust.png"),width=2000,height=1000)
    plot(DoHeatmap(sc10x,genes.use=gs,slim.col.label=TRUE,group.label.rot=TRUE,group.spacing=0.25,cex.row=2.5,group.order=g))
    dev.off()
  },error=function(e){cat("ERROR : ",conditionMessage(e),"/\n")})
}


scPseudotime <- function(sc10x,i,ds){
  sc10x <- SetAllIdent(object=sc10x,id=i)
  
  if (ncol(sc10x@data)<ds){
    rnd <- sample(1:ncol(sc10x@data),ds)
    sc10x <- SetIdent(object=sc10x,cells.use=sc10x@data@Dimnames[[2]][rnd],ident.use="sample")
    sc10x <- SubsetData(object=sc10x,ident.use="samples")
    sc10x <- SetAllIdent(object=sc10x,id=i)
  }
  scMonocle <- importCDS(sc10x)
  scMonocle <- estimateSizeFactors(scMonocle)
  scMonocle <- estimateDispersions(scMonocle)
  
  DEG <- differentialGeneTest(scMonocle,fullModelFormulaStr=paste0("~",i),cores=50)
  my_ordering_genes <- row.names(DEG)[order(DEG$qval)][1:1000]
  scMonocle <- setOrderingFilter(scMonocle,ordering_genes=my_ordering_genes)
  scMonocle <- reduceDimension(scMonocle,method='DDRTree')
  scMonocle <- orderCells(scMonocle)
  postscript(paste0("./analysis/pseudotime/Trajectory.State.eps"),paper="special",width=10,height=10,horizontal=FALSE)
  plot(plot_cell_trajectory(scMonocle,color_by="State"))
  dev.off()
  postscript(paste0("./analysis/pseudotime/Trajectory.",i,".eps"),paper="special",width=10,height=10,horizontal=FALSE)
  plot(plot_cell_trajectory(scMonocle,color_by=i))
  dev.off()
  
  my_pseudotime_de <- differentialGeneTest(scMonocle,fullModelFormulaStr = "~sm.ns(Pseudotime)",cores=50)
  my_pseudotime_de %>% arrange(qval) %>% head(10) %>% select(gene_short_name) -> my_pseudotime_gene
  my_pseudotime_gene <- my_pseudotime_gene$gene_short_name
  my_pseudotime_gene <- as.character(my_pseudotime_gene)
  postscript(paste0("./analysis/pseudotime/Genes.State.eps"),paper="special",width=10,height=10,horizontal=FALSE)
  plot(plot_genes_branched_pseudotime(scMonocle[my_pseudotime_gene,],color_by="State"))
  dev.off()
  postscript(paste0("./analysis/pseudotime/Genes.",i,".eps"),paper="special",width=10,height=10,horizontal=FALSE)
  plot(plot_genes_branched_pseudotime(scMonocle[my_pseudotime_gene,],color_by=i))
  dev.off()
  
  
  my_pseudotime_de %>% arrange(qval) %>% head(50) %>% select(gene_short_name) -> gene_to_cluster
  gene_to_cluster <- gene_to_cluster$gene_short_name
  postscript(paste0("./analysis/pseudotime/Heatmap.eps"),paper="special",width=10,height=10,horizontal=FALSE)
  my_pseudotime_cluster <- plot_pseudotime_heatmap(scMonocle[gene_to_cluster,],show_rownames=TRUE,return_heatmap=TRUE)
  dev.off()
  
  #BEAM_res <- BEAM(scMonocle,branch_point=1,cores=50)
  #BEAM_res <- BEAM_res[order(BEAM_res$qval),]
  #BEAM_res <- BEAM_res[,c("gene_short_name","pval","qval")]
  #plot_genes_branched_heatmap(scMonocle[rownames(subset(BEAM_res,qval<1e-4)),],branch_point=1,num_clusters=5,cores=50,use_gene_short_name=TRUE,show_rownames=TRUE)
}


RenameCells <- function(object, add.cell.id = NULL, new.names = NULL,
                        for.merge = FALSE) {
  
  if (missing(add.cell.id) && missing(new.names)) {
    stop("One of 'add.cell.id' and 'new.names' must be set")
  }
  
  if (!missing(add.cell.id) && !missing(new.names)) {
    stop("Only one of 'add.cell.id' and 'new.names' must be set")
  }
  
  if (!missing(add.cell.id)) {
    new.cell.names <- paste(add.cell.id, object@cell.names, sep = "_")
    new.rawdata.names <- paste(add.cell.id, colnames(object@raw.data), sep = "_")
  } else {
    if(ncol(object@raw.data) != ncol(object@data)) {
      stop("raw.data contains a different number of cells than data")
    }
    if(any(colnames(object@raw.data) != colnames(object@data))){
      stop("cells in raw.data are different than the cells in data")
    }
    if (length(new.names) == length(object@cell.names)) {
      new.cell.names <- new.names
      new.rawdata.names <- new.names
    } else {
      stop("the length of 'new.names' (", length(new.names), ") must be the ",
           "same as the length of 'object@cell.names' (",
           length(object@cell.names), ")")
    }
  }
  colnames(object@raw.data) <- new.rawdata.names
  rownames(object@meta.data) <- new.cell.names
  object@cell.names <- new.cell.names
  
  if (for.merge) {
    return(object)
  }
  
  colnames(object@data) <- new.cell.names
  
  if (!is.null(object@scale.data)) {
    colnames(object@scale.data) <- new.cell.names
  }
  names(object@ident) <- new.cell.names
  
  if (length(object@dr) > 0) {
    for (dr in names(object@dr)) {
      rownames(object@dr[[dr]]@cell.embeddings) <- new.cell.names
    }
  }
  
  if (nrow(object@snn) == length(new.cell.names)) {
    colnames(object@snn) <- new.cell.names
    rownames(object@snn) <- new.cell.names
  }
  
  if (!is.null(object@kmeans)) {
    if (!is.null(object@kmeans@gene.kmeans.obj)) {
      colnames(object@kmeans@gene.kmeans.obj$centers) <- new.cell.names
    }
    if (!is.null(object@kmeans@cell.kmeans.obj)) {
      names(object@kmeans@cell.kmeans.obj$cluster) <- new.cell.names
    }
  }
  
  # rownames(object@spatial@mix.probs) <- new.cell.names
  
  return(object)
}

